--This addon responds to incoming whispers with an outgoing "do not disturb" message
--The message can be altered or set back to the default
--The standard behavior is for return whispers to be echoed to the chat window, but that can be disabled

--#######################################################
--Variable and table initialization
--#######################################################

ajdndVer = "1.0";
ajdndStatus = "OFF";	--Must be re-enabled each login, to avoid inadvertently leaving it ON (so not a saved variable)
ajdndEcho = "ON";		--Corresponds with default system behavior, outgoing whispers are echoed (this is a saved variable)
ajdndToggled = 0;		--Only display instructions the first close per session (not a saved variable)

--Table to hold message parts
ajdndmsg = {

header = [[AmperDND: ]],

default = [[I'm currently busy. Please try again later, or send me a message in WoW email. Thanks!]],

current = [[]],

}

ajdnd_MINIMAP_TOOLTIP_FOOTER = "Shift+click or right-click to move\nAlt+Shift+click for free drag and drop";

--#######################################################
--Function definitions
--#######################################################

--Can choose whether to see the outgoing replies or not (the default behavior is that they are visible)
local function HideOutgoing(self, event, msg, author, ...)
  if string.find(msg, "AmperDND") then
	--print("Do not disturb message sent" .. msg)
	return true
  end
end

function AJDNDPlayTickSound()
	PlaySound("igMainMenuOptionCheckBoxOn")
end

--#######################################################
--Button handlers in the AmperDND window
--#######################################################
--Called by clicking the ON button
--Since the addon is always off by default at log-in, put the check of saved variables here
--(rather than at load time which was sometimes checking before the variable was actually loaded)
function enableAJDND()
	if (ajDNDcustomMsg ~= nil) then
		print("re-loaded saved custom message; ajDNDcustomMsg = " .. ajDNDcustomMsg);
		ajdndmsg.current = ajDNDcustomMsg
	else
		print("didn't load custom message; ajDNDcustomMsg apparently nil, applied default");
		ajdndmsg.current = ajdndmsg.default
	end
	ajdndStatus = "ON";
	local display = "Outgoing message:|n" .. ajdndmsg.current
	AmperDND.text:Hide();
	AmperDND.text:SetText(display);
	AmperDND.text:Show();
end

--Called by clicking the OFF button
function disableAJDND()
	ajdndStatus = "OFF";
	AmperDND.text:Hide();
	AmperDND.text:SetText("disabled");
	AmperDND.text:Show();
end

--Called by the EDIT button
function openAJDNDReset()
	ResetAwayMsg:Show();
	AJDNDPlayTickSound();
end

--Called by the ECHO button, toggles this setting on/off
function setAJDNDEcho()
	--Default is ON
	if (ajdndEcho == "OFF") then
		ajdndEcho = "ON";
		--Set display to indicate echo is ON
		AmperDND.echo:SetText("Echo outgoing whispers: ON");
		ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", HideOutgoing)
		ChatFrame_RemoveMessageEventFilter("CHAT_MSG_BN_WHISPER_INFORM", HideOutgoing)
	elseif (ajdndEcho == "ON") then
		ajdndEcho = "OFF";
		--Set display to indicate echo is OFF
		AmperDND.echo:SetText("Echo outgoing whispers: OFF");
		--Then, if the addon is actually turned on...
		if (ajdndStatus == "ON") then
			ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", HideOutgoing)
			ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER_INFORM", HideOutgoing)
		end
	end
end

--Called by clicking the CLOSE button or toggling with the Ampersand graphic
function closeAJDND()
	--print("entered function closeAJDND")
	AmperDND:Hide();
	if ((ajdndToggled == 0) or (ajdndToggled == nil)) then
		--print("ajdndToggled = " .. ajdndToggled);
		print("|cffff8000AmperDND|r: Closed settings window; click the Ampersand to open again");
		ajdndToggled = 1;
		--print("now ajdndToggled = " .. ajdndToggled);
	end
end




--#######################################################
--Button handlers in the ResetAwayMsg window
--#######################################################

--Called by the APPLY button (entering a new message)
function applyAJDNDUpdate()
	local newAwayMsg = ResetAwayMsg.input:GetText();
	print("|cffff8000AmperDND|r: New away message: " .. newAwayMsg);
	ajdndmsg.current = newAwayMsg;
	--and remove the previous value from the saved variable
	ajDNDcustomMsg = ajdndmsg.current;
	ResetAwayMsg:Hide();
	AmperDND.text:SetText("Outgoing message:|n" .. newAwayMsg);
	AJDNDPlayTickSound()
end

--Called by the DEFAULT button (returning to the default message)
function defaultAJDNDUpdate()
	print("|cffff8000AmperDND|r: Returned away message to default: " .. ajdndmsg.default);
	--and remove the previous value from the saved variable
	ajdndmsg.current = ajdndmsg.default;
	ajDNDcustomMsg = nil;
	ResetAwayMsg:Hide();
	local display = "Outgoing message:|n" .. ajdndmsg.current
	AmperDND.text:Hide();
	AmperDND.text:SetText(display);
	AmperDND.text:Show();
	AJDNDPlayTickSound();
end

--Called by the CANCEL button (closing the edit window without changes)
function cancelAJDNDUpdate()
	ResetAwayMsg:Hide();
	AJDNDPlayTickSound();
end

--#######################################################
--When the addon loads
--#######################################################
print(" ");
print("|cffff8000AmperDND|r: loaded successfully; off by default; click the & to open");
print(" ");


--#######################################################
--Handle regular whisper events
--#######################################################
local ChatListenFrame = CreateFrame("Frame");

ChatListenFrame:RegisterEvent("CHAT_MSG_WHISPER");
ChatListenFrame:RegisterEvent("CHAT_MSG_WHISPER_INFORM");

ChatListenFrame:SetScript("OnEvent", function(event, ...)

	local eType = select(1, ...); -- eventType
	local msg = select(2, ...); -- message
	local sender = select(3, ...); -- sender
	--print("A chat was noticed: " .. eType .. ", " .. msg .. ", " .. sender);


	if (eType == "CHAT_MSG_WHISPER") then
		if (ajdndStatus == "ON") then
			local receivedFrom = sender;
			local output = ajdndmsg.header .. ajdndmsg.current
			--print("should be sending " .. output .. " to " .. receivedFrom);
			SendChatMessage(output, "WHISPER", nil, receivedFrom)
		end
	end

end)

--#######################################################
--Handle Battle.net (RealID) whisper events
--#######################################################
local BNChatListenFrame = CreateFrame("Frame");

BNChatListenFrame:RegisterEvent("CHAT_MSG_BN_WHISPER");
BNChatListenFrame:RegisterEvent("CHAT_MSG_BN_WHISPER_INFORM");

BNChatListenFrame:SetScript("OnEvent", function(event, ...)

	local eType = select(1, ...); -- eventType
	local msg = select(2, ...); -- message
	local sender = select(3, ...); -- sender
	local pID = select(14, ...); --presenceID of the battle.net friend (the source of the whisper)
	--print("A battle.net chat was noticed: " .. eType .. ", " .. msg .. ", " .. sender .. ", " .. pID);


	if (eType == "CHAT_MSG_BN_WHISPER") then
		if (ajdndStatus == "ON") then
			local output = ajdndmsg.header .. ajdndmsg.current
			BNSendWhisper(pID, output)
		end
	end

end)


--#######################################################
--The minimap button
--#######################################################
do
	local dragMode = nil

	local function moveButton(self)
		if dragMode == "free" then
			local centerX, centerY = Minimap:GetCenter()
			local x, y = GetCursorPosition()
			x, y = x / self:GetEffectiveScale() - centerX, y / self:GetEffectiveScale() - centerY
			self:ClearAllPoints()
			self:SetPoint("CENTER", x, y)
		else
			local centerX, centerY = Minimap:GetCenter()
			local x, y = GetCursorPosition()
			x, y = x / self:GetEffectiveScale() - centerX, y / self:GetEffectiveScale() - centerY
			centerX, centerY = math.abs(x), math.abs(y)
			centerX, centerY = (centerX / math.sqrt(centerX^2 + centerY^2)) * 80, (centerY / sqrt(centerX^2 + centerY^2)) * 80
			centerX = x < 0 and -centerX or centerX
			centerY = y < 0 and -centerY or centerY
			self:ClearAllPoints()
			self:SetPoint("CENTER", centerX, centerY)
		end
	end

	local button = CreateFrame("Button", "ADNDMinimapButton", Minimap)
	button:SetHeight(32)
	button:SetWidth(32)
	button:SetFrameStrata("MEDIUM")
	button:SetPoint("CENTER", -65.35, -38.8)
	button:SetMovable(true)
	button:SetUserPlaced(true)
	button:SetNormalTexture("Interface\\AddOns\\AmperDND\\graphics\\Amperbutton-Up")
	button:SetPushedTexture("Interface\\AddOns\\AmperDND\\graphics\\Amperbutton-Down")
	button:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")

	button:SetScript("OnMouseDown", function(self, button)
		if IsShiftKeyDown() and IsAltKeyDown() then
			dragMode = "free"
			self:SetScript("OnUpdate", moveButton)
		elseif IsShiftKeyDown() or button == "RightButton" then
			dragMode = nil
			self:SetScript("OnUpdate", moveButton)
		end
	end)
	button:SetScript("OnMouseUp", function(self)
		self:SetScript("OnUpdate", nil)
	end)
	button:SetScript("OnClick", function(self, button)
		if IsShiftKeyDown() or button == "RightButton" then return end
		if (AmperDND:IsShown() == 1) then
			closeAJDND();
		elseif (AmperDND:IsShown() == nil) then
			AmperDND:Show();
		end;
	end)
	--This is the mouse-over script...
	button:SetScript("OnEnter", function(self)
		GameTooltip_SetDefaultAnchor(GameTooltip, self)
		GameTooltip:SetText("Amper Do Not Disturb", 1, 1, 1)
		GameTooltip:AddLine("v." .. ajdndVer, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1) -- revert to 1, 1, 1, 1 for white?
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(ajdnd_MINIMAP_TOOLTIP_FOOTER, RAID_CLASS_COLORS.MAGE.r, RAID_CLASS_COLORS.MAGE.g, RAID_CLASS_COLORS.MAGE.b, 1)
		GameTooltip:Show()
	end)
	button:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)

end
